package com.tanpu.community.manager;

import com.tanpu.common.api.CommonResp;
import com.tanpu.common.auth.UserHolder;
import com.tanpu.common.constant.ErrorCodeConstant;
import com.tanpu.community.api.beans.qo.TopicFollowQo;
import com.tanpu.community.api.beans.qo.TopicRankQo;
import com.tanpu.community.api.beans.req.page.Page;
import com.tanpu.community.api.beans.req.topic.FollowTopicReq;
import com.tanpu.community.api.beans.req.topic.TopicSearchReq;
import com.tanpu.community.api.enums.OperationTypeEnum;
import com.tanpu.community.api.enums.TopicSpecialPermissionEnum;
import com.tanpu.community.dao.entity.community.TopicEntity;
import com.tanpu.community.dao.entity.community.TopicFollowRelEntity;
import com.tanpu.community.dao.mapper.community.TopicFollowRelMapper;
import com.tanpu.community.service.RankService;
import com.tanpu.community.service.ThemeService;
import com.tanpu.community.service.TopicService;
import com.tanpu.community.util.PageUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class TopicManager {


    @Resource
    private TopicService topicService;
    @Resource
    private RankService rankService;

    @Resource
    private UserHolder userHolder;

    @Resource
    TopicFollowRelMapper topicFollowRelMapper;
    @Resource
    private ThemeService themeService;

    // 首页-话题标签
    public List<TopicRankQo> getTop4TopicTitles() {
        return rankService.getRankTopicListTop4();
    }


    // 话题搜索列表
    public Page<TopicRankQo> getAllTopicBriefInfo(TopicSearchReq req) {
        // 全量排序，内存分页
        List<TopicRankQo> topicList = rankService.getRankTopicList(req.getSearchKeyword());
        Page<TopicRankQo> result = PageUtils.page(req.getPage(), topicList);
        // 添加权限
        topicService.batchCheckPermission(result.getContent(),userHolder.getUserId());
        return result;
    }

    /**
     * 关注话题（讨论区）列表
     */

    public List<TopicFollowQo> getFollowTopicList() {

        String userId = userHolder.getUserId();
        // 查库
        List<TopicFollowQo> topicFollowQos = topicService.queryFollowTopic(userId);
        // 先从缓存中获取浏览量
        Map<String, TopicRankQo> topicMap = rankService.getRankTopicList(null).stream().collect(Collectors.toMap(TopicRankQo::getTopicId, o -> o, (a, b) -> a));

        topicFollowQos.stream().forEach(o -> {
            TopicRankQo topicRankQo = topicMap.get(o.getTopicId());
            BeanUtils.copyProperties(topicRankQo, o);
        });


        // 最新讨论
        themeService.queryCommentForTopic(topicFollowQos, userId);

        // 排序
        topicFollowQos.stream().sorted(Comparator.comparing(TopicFollowQo::getSpecialPermission, Comparator.reverseOrder()).
                thenComparing(TopicFollowQo::getLastThemeTime, Comparator.nullsFirst(String::compareTo).reversed()))
                .collect(Collectors.toList());

        return topicFollowQos;
    }

    /**
     * 话题详情页
     *
     * @param topicId
     * @return
     */
    public CommonResp<TopicRankQo> getDetail(String topicId) {


        TopicEntity topicEntity = topicService.queryOnlineTopicById(topicId);
        if (topicEntity == null) {
            return CommonResp.error(ErrorCodeConstant.TOPIC_NOT_FOUND.getCode(), "抱歉！该话题已下线。");
        }

        //
        if (TopicSpecialPermissionEnum.TRUE.getCode().equals(topicEntity.getSpecialPermission()) && !checkPermission(topicId, userHolder.getUserId())) {
            return CommonResp.error(ErrorCodeConstant.TOPIC_PERMISSION_ABORT.getCode(), getPermissionToast(topicId));
        }

        return CommonResp.success(rankService.getTopicDetail(topicId));
    }

    private String getPermissionToast(String topicId) {

        String permission = "专业版会员";

        return "暂无权限参与此话题～您可联系官方客服 021- 了解详情";
        // return "该话题仅限" + permission + "可参与哦~";
    }

    /**
     * 判断用户是否拥有权限（1对1）
     *
     * @param topicId
     * @param userId
     */
    private boolean checkPermission(String topicId, String userId) {

        TopicEntity topicEntity = topicService.queryOnlineTopicById(topicId);
        if (TopicSpecialPermissionEnum.FALSE.equals(topicEntity.getSpecialPermission())) {
            return true;
        }
        TopicFollowRelEntity topicFollowRelEntity = topicFollowRelMapper.queryOneByTopicIdAndUserId(topicId, userId);
        if (topicFollowRelEntity != null) {
            return true;
        } else {
            return false;
        }
    }


    /**
     * 关注/取关 话题
     *
     * @param req
     * @param userId
     * @return
     */
    public CommonResp<Void> followTopic(FollowTopicReq req, String userId) {

        // 检验话题权限是否专属
        TopicEntity topicEntity = topicService.queryOnlineTopicById(req.getTopicId());
        if (TopicSpecialPermissionEnum.TRUE.equals(topicEntity.getSpecialPermission())) {
            return CommonResp.error(ErrorCodeConstant.ILLEGAL_ARGEMENT.getCode(), "专属话题不支持关注");
        }

        if (OperationTypeEnum.CONFIRM.getCode().equals(req.getType())) {
            topicService.addFollowTopic(req.getTopicId(), userId);
        } else if (OperationTypeEnum.CANCEL.getCode().equals(req.getType())) {
            topicService.deleteFollowTopic(req.getTopicId(), userId);
        }
        return CommonResp.success();
    }
}

